@@ -10,6 +10,7 @@ use std::ops::{Add, AddAssign, Mul, Neg, Sub};
 use cumsum::cumsum_array;
 use daggy::{Dag, WouldCycle};
 use daggy::petgraph::data::Element;
+use index_utils::remove_indices;
 use itertools::repeat_n;
 use num_traits::Zero;
 use relp_num::{OrderedField, OrderedFieldRef};
@@ -18,7 +19,6 @@ use relp_num::NonZero;
 pub use presolve::scale::{Scalable, Scaling};
 
 use crate::algorithm::two_phase::matrix_provider::matrix_data::MatrixData;
-use crate::algorithm::utilities::remove_indices;
 use crate::data::linear_algebra::matrix::{ColumnMajor, MatrixOrder};
 use crate::data::linear_algebra::matrix::SparseMatrix;
 use crate::data::linear_algebra::SparseTupleVec;
@@ -261,16 +261,19 @@ where
     ///
     /// In case the linear program gets solved during this presolve operation, a solution.
     pub fn derive_matrix_data(&self, constraint_type_counts: [usize; 4]) -> MatrixData<OF> {
-        debug_assert!('check: {
+        debug_assert!({
             let boundaries = cumsum_array(&constraint_type_counts);
-            for (i, ct) in self.constraint_types.iter().enumerate() {
-                use RangedConstraintRelation::*;
-                     if i < boundaries[0] { if !matches!(ct,    Equal) { break 'check false; } }
-                else if i < boundaries[1] { if !matches!(ct, Range(_)) { break 'check false; } }
-                else if i < boundaries[2] { if !matches!(ct,     Less) { break 'check false; } }
-                else if i < boundaries[3] { if !matches!(ct,  Greater) { break 'check false; } }
-            }
-            true
+            self.constraint_types.iter().enumerate()
+                .all(|(i, constraint_type)| {
+                    use RangedConstraintRelation::*;
+                         if i < boundaries[0] { matches!(constraint_type,    Equal) }
+                    else if i < boundaries[1] { matches!(constraint_type, Range(_)) }
+                    else if i < boundaries[2] { matches!(constraint_type,     Less) }
+                    else if i < boundaries[3] { matches!(constraint_type,  Greater) }
+                    else {
+                        panic!();
+                    }
+                })
         });
 
         let [
@@ -365,7 +368,7 @@ where
             removed_variables,
         );
         self.remove_rows_and_columns(constraints_marked_removed, variable_indices_only);
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
 
         self.compute_solution_where_possible();
         self.get_solution()
@@ -384,7 +387,7 @@ where
     fn compute_presolve_changes(
         &self
     ) -> Result<Changes<OF>, LinearProgramType<OF>> {
-        let mut index = PresolveIndex::new(&self)?;
+        let mut index = PresolveIndex::new(self)?;
 
         let mut iterations_without_meaningful_change = 0;
         while !index.are_queues_empty() && iterations_without_meaningful_change < {
@@ -490,7 +493,7 @@ where
         remove_indices(&mut self.constraint_types, &constraints);
         self.b.remove_indices(&constraints);
 
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
     }
 
     /// Split free variables and shift all variables, such that the lower bound is zero.
@@ -504,7 +507,7 @@ where
     fn transform_variables(&mut self) {
         // After this call, all active variables have at least one bound.
         self.split_free_variables();
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
 
         for j in 0..self.variables.len() {
             let variable = &mut self.variables[j];
@@ -540,7 +543,7 @@ where
             }
         }
 
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
     }
 
     /// Split all free variables.
@@ -614,7 +617,7 @@ where
             };
         }
 
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
     }
 
     /// Make this a minimization problem by multiplying the cost function by `-1`.
@@ -708,7 +711,7 @@ where
             column.sort_unstable_by_key(|&(i, _)| i);
         }
 
-        debug_assert!(is_consistent(&self));
+        debug_assert!(is_consistent(self));
 
         [e_counter, r_counter, l_counter, g_counter]
     }
@@ -848,7 +851,7 @@ where
         debug_assert_eq!(reduced_solution.len(), self.variables.len());
 
         let cost = reduced_solution.iter()
-            .map(|(j, v)| v * &self.variables[*j].cost)
+            .map(|(j, v)| v * &self.variables[j].cost)
             .sum::<G>() + &self.fixed_cost;
         self.reshift_solution(&mut reduced_solution);
 
